import sys
import shutil
import os
import glob
from subprocess import call
import re

def tryint(s):
    try:
        return int(s)
    except:
        return s

def alphanum_key(s):
    """ Turn a string into a list of string and number chunks.
        "z23a" -> ["z", 23, "a"]
    """
    return [ tryint(c) for c in re.split('([0-9]+)', s) ]

# THE PROGRAM BEGINS HERE

if (len(sys.argv) < 4):
    print "Usage: prepare_dataset.py input_dir out_dir original_extension";
    sys.exit();

frame_path = sys.argv[1];
output_path = sys.argv[2];
extension = sys.argv[3];
frame_output_path = os.path.join(output_path, 'frames');
flow_output_path = os.path.join(output_path, 'flows');

#create directories needed
if (not os.path.exists(output_path)):
    os.makedirs(output_path);

if (not os.path.exists(frame_output_path)):
    os.makedirs(frame_output_path);

if (not os.path.exists(flow_output_path)):
    os.makedirs(flow_output_path);

#list all the frames - the program sorts them automatically
files_to_search = os.path.join(frame_path, "[!.]*." + extension);
frames = glob.glob(files_to_search);
frames.sort(key=alphanum_key);

print "Computing the optical flow for ", len(frames), " frames";

#convert all the frames to ppm (required by LDOF). Be carefull not to overwrite frames
k = 0;
for i in range(len(frames)):
    input_frame_name = frames[i];
    output_frame_name = "frame" + str(i).zfill(3) + ".png";
    output_frame_name_ppm = "frame" + str(i).zfill(3) + ".ppm";
    call("convert " + input_frame_name + " " + os.path.join(frame_output_path,output_frame_name), shell=True);
    call("convert " + input_frame_name + " " + os.path.join(frame_output_path,output_frame_name_ppm), shell=True);

#compute forward and backward flows
for i in range(len(frames)-1):
    print "Computing flow between ", i, "and", i+1;
    frame1_name =  os.path.join(frame_output_path,"frame" + str(i).zfill(3) + ".ppm");
    frame2_name =  os.path.join(frame_output_path,"frame" + str(i+1).zfill(3) + ".ppm");
    
    #optical flow files
    frame1_ldof_flo = os.path.join(frame_output_path,"frame" + str(i).zfill(3) + "LDOF.flo");
    frame2_ldof_flo = os.path.join(frame_output_path,"frame" + str(i+1).zfill(3) + "LDOF.flo");
    output1_ldof = os.path.join(flow_output_path,"flow_" + str(i) + "_" + str(i+1) + ".flo");
    output2_ldof = os.path.join(flow_output_path,"flow_" + str(i+1) + "_" + str(i) + ".flo");
    
    #image files
    frame1_ldof_ppm =  os.path.join(frame_output_path,"frame" + str(i).zfill(3) + "LDOF.ppm");
    frame2_ldof_ppm =  os.path.join(frame_output_path,"frame" + str(i+1).zfill(3) + "LDOF.ppm");
    frame1_ldof_png =  os.path.join(flow_output_path,"motion_" + str(i) + "_" + str(i+1) + ".png");
    frame2_ldof_png =  os.path.join(flow_output_path,"motion_" + str(i+1) + "_" + str(i) + ".png");
    
    #call LDOF
    call("./ldof " + frame1_name + " " + frame2_name, shell=True);
    call("./ldof " + frame2_name + " " + frame1_name, shell=True);
    
    #remove and move files created to corresponding locations
    shutil.move(frame1_ldof_flo, output1_ldof);
    shutil.move(frame2_ldof_flo, output2_ldof);
    
    call("convert " + frame1_ldof_ppm + " " + frame1_ldof_png, shell=True);
    call("convert " + frame2_ldof_ppm + " " + frame2_ldof_png, shell=True);
    os.unlink(frame1_ldof_ppm);
    os.unlink(frame2_ldof_ppm);
    
#remove the ppm files in the frames folder
for f in glob.glob(os.path.join(frame_output_path,"*.ppm")):
    os.unlink(f);
